@@ -1,5 +1,3 @@
-/* $XdotOrg: app/xdm/xdmcp.c,v 1.6 2006/06/20 01:50:35 alanc Exp $ */
-/* $Xorg: xdmcp.c,v 1.4 2001/02/09 02:05:41 xorgcvs Exp $ */
 /*
 
 Copyright 1988, 1998  The Open Group
@@ -27,7 +25,6 @@ other dealings in this Software without prior written authorization
 from The Open Group.
 
 */
-/* $XFree86: xc/programs/xdm/xdmcp.c,v 3.25 2003/11/23 22:36:03 herrb Exp $ */
 
 /*
  * xdm - display manager daemon
@@ -36,9 +33,9 @@ from The Open Group.
  * xdmcp.c - Support for XDMCP
  */
 
-# include "dm.h"
-# include "dm_auth.h"
-# include "dm_error.h"
+#include "dm.h"
+#include "dm_auth.h"
+#include "dm_error.h"
 
 #ifdef XDMCP
 
@@ -49,20 +46,16 @@ from The Open Group.
 
 # include	"dm_socket.h"
 
-#ifndef X_NO_SYS_UN
-#ifndef Lynx
-#include	<sys/un.h>
-#else
-#include	<un.h>
-#endif
-#endif
-#include	<netdb.h>
-#include	<arpa/inet.h>
+# ifndef X_NO_SYS_UN
+#  include	<sys/un.h>
+# endif
+# include	<netdb.h>
+# include	<arpa/inet.h>
 
-#include <time.h>
-#define Time_t time_t
+# include <time.h>
+# define Time_t time_t
 
-#define getString(name,len)	((name = malloc (len + 1)) ? 1 : 0)
+# define getString(name,len)	((name = malloc (len + 1)) ? 1 : 0)
 
 /*
  * misc externs
@@ -86,44 +79,44 @@ static void send_refuse (struct sockaddr *from, int fromlen, CARD32 sessionID, i
 static void send_unwilling (struct sockaddr *from, int fromlen, ARRAY8Ptr authenticationName, ARRAY8Ptr status, int fd);
 static void send_willing (struct sockaddr *from, int fromlen, ARRAY8Ptr authenticationName, ARRAY8Ptr status, int fd);
 
-#ifdef STREAMSCONN
+# ifdef STREAMSCONN
 int	xdmcpFd = -1;
-#endif
+# endif
 int	chooserFd = -1;
-#if defined(IPv6) && defined(AF_INET6)
+# if defined(IPv6) && defined(AF_INET6)
 int	chooserFd6 = -1;
-#endif
+# endif
 
 FD_TYPE	WellKnownSocketsMask;
 int	WellKnownSocketsMax;
 
-#define pS(s)	((s) ? ((char *) (s)) : "empty string")
+# define pS(s)	((s) ? ((char *) (s)) : "empty string")
 
 void
 DestroyWellKnownSockets (void)
 {
-#ifdef STREAMSCONN
+# ifdef STREAMSCONN
     if (xdmcpFd != -1)
     {
 	close (xdmcpFd);
 	FD_CLR(xdmcpFd, &WellKnownSocketsMask);
 	xdmcpFd = -1;
     }
-#endif
+# endif
     if (chooserFd != -1)
     {
 	close (chooserFd);
 	FD_CLR(chooserFd, &WellKnownSocketsMask);
 	chooserFd = -1;
     }
-#if defined(IPv6) && defined(AF_INET6)
+# if defined(IPv6) && defined(AF_INET6)
     if (chooserFd6 != -1)
     {
 	close (chooserFd6);
 	FD_CLR(chooserFd6, &WellKnownSocketsMask);
 	chooserFd6 = -1;
     }
-#endif
+# endif
     CloseListenSockets();
 }
 
@@ -143,13 +136,13 @@ FD_ANYSET(fd_set *fds)
 int
 AnyWellKnownSockets (void)
 {
-    return 
-#ifdef STREAMS_CONN
+    return
+# ifdef STREAMS_CONN
       xdmcpFd != -1 ||
-#endif
-#if defined(IPv6) && defined(AF_INET6)
+# endif
+# if defined(IPv6) && defined(AF_INET6)
       chooserFd6 != -1 ||
-#endif
+# endif
       chooserFd != -1 || FD_ANYSET(&WellKnownSocketsMask);
 }
 
@@ -162,26 +155,26 @@ sendForward (
     ARRAY8Ptr	address,
     char	*closure)
 {
-#ifdef AF_INET
+# ifdef AF_INET
     struct sockaddr_in	    in_addr;
-#endif
-#if defined(IPv6) && defined(AF_INET6)
+# endif
+# if defined(IPv6) && defined(AF_INET6)
     struct sockaddr_in6	    in6_addr;
-#endif
-#ifdef AF_DECnet
-#endif
+# endif
+# ifdef AF_DECnet
+# endif
     struct sockaddr	    *addr;
     int			    addrlen;
 
     switch (connectionType)
     {
-#ifdef AF_INET
+# ifdef AF_INET
     case FamilyInternet:
 	addr = (struct sockaddr *) &in_addr;
 	bzero ((char *) &in_addr, sizeof (in_addr));
-#ifdef BSD44SOCKETS
+#  ifdef BSD44SOCKETS
 	in_addr.sin_len = sizeof(in_addr);
-#endif
+#  endif
 	in_addr.sin_family = AF_INET;
 	in_addr.sin_port = htons ((short) XDM_UDP_PORT);
 	if (address->length != 4)
@@ -189,14 +182,14 @@ sendForward (
 	memmove( (char *) &in_addr.sin_addr, address->data, address->length);
 	addrlen = sizeof (struct sockaddr_in);
 	break;
-#endif
-#if defined(IPv6) && defined(AF_INET6)
+# endif
+# if defined(IPv6) && defined(AF_INET6)
     case FamilyInternet6:
 	addr = (struct sockaddr *) &in6_addr;
 	bzero ((char *) &in6_addr, sizeof (in6_addr));
-#ifdef SIN6_LEN
+#  ifdef SIN6_LEN
 	in6_addr.sin6_len = sizeof(in6_addr);
-#endif
+#  endif
 	in6_addr.sin6_family = AF_INET6;
 	in6_addr.sin6_port = htons ((short) XDM_UDP_PORT);
 	if (address->length != 16)
@@ -204,10 +197,10 @@ sendForward (
 	memmove( (char *) &in6_addr.sin6_addr, address->data, address->length);
 	addrlen = sizeof (struct sockaddr_in6);
 	break;
-#endif
-#ifdef AF_DECnet
+# endif
+# ifdef AF_DECnet
     case FamilyDECnet:
-#endif
+# endif
     default:
 	return;
     }
@@ -247,20 +240,20 @@ all_query_respond (
     int			fd)
 {
     ARRAY8Ptr	authenticationName;
-    ARRAY8	status;
-    ARRAY8	addr;
+    ARRAY8	status = {0, NULL};
+    ARRAY8	addr  = {0, NULL};
     CARD16	connectionType;
     int		family;
     int		length;
     const char	*addrstring;
-#if defined(IPv6) && defined(AF_INET6) 
+# if defined(IPv6) && defined(AF_INET6)
     char	addrbuf[INET6_ADDRSTRLEN] = "";
-#endif
+# endif
 
     family = ConvertAddr((XdmcpNetaddr) from, &length, (char **)&(addr.data));
     addr.length = length;	/* convert int to short */
     if (debugLevel > 0) {
-#if defined(IPv6) && defined(AF_INET6) 
+# if defined(IPv6) && defined(AF_INET6)
 	void *ipaddr;
 	int af_type;
 	if (family == FamilyInternet6) {
@@ -271,9 +264,9 @@ all_query_respond (
 	    af_type = AF_INET;
 	}
 	addrstring = inet_ntop(af_type, ipaddr, addrbuf, sizeof(addrbuf));
-#else
+# else
 	addrstring = inet_ntoa(((struct sockaddr_in *)from)->sin_addr);
-#endif
+# endif
 	Debug("all_query_respond: conntype=%d, addr=%s, len=%d\n",
 	    family, addrstring, addr.length);
     }
@@ -302,15 +295,15 @@ indirect_respond (
     int		    length,
     int		    fd)
 {
-    ARRAYofARRAY8   queryAuthenticationNames;
-    ARRAY8	    clientAddress;
-    ARRAY8	    clientPort;
+    ARRAYofARRAY8   queryAuthenticationNames = {0, NULL};
+    ARRAY8	    clientAddress = {0, NULL};
+    ARRAY8	    clientPort = {0, NULL};
     CARD16	    connectionType;
     int		    expectedLen;
     int		    i;
     XdmcpHeader	    header;
     int		    localHostAsWell;
-    
+
     Debug ("Indirect respond %d\n", length);
     if (!XdmcpReadARRAYofARRAY8 (&buffer, &queryAuthenticationNames))
 	return;
@@ -337,7 +330,7 @@ indirect_respond (
     	XdmcpWriteARRAYofARRAY8 (&buffer, &queryAuthenticationNames);
 
 	localHostAsWell = ForEachMatchingIndirectHost (&clientAddress, connectionType, sendForward, (char *) &fd);
-	
+
 	XdmcpDisposeARRAY8 (&clientAddress);
 	XdmcpDisposeARRAY8 (&clientPort);
 	if (localHostAsWell)
@@ -355,11 +348,11 @@ void
 ProcessRequestSocket (int fd)
 {
     XdmcpHeader		header;
-#if defined(IPv6) && defined(AF_INET6)
+# if defined(IPv6) && defined(AF_INET6)
     struct sockaddr_storage	addr;
-#else
+# else
     struct sockaddr	addr;
-#endif
+# endif
     int			addrlen = sizeof addr;
 
     Debug ("ProcessRequestSocket\n");
@@ -418,32 +411,32 @@ WaitForSomething (void)
 		nready, Rescan, ChildReady);
 	if (nready > 0)
 	{
-#ifdef STREAMSCONN
+# ifdef STREAMSCONN
 	    if (xdmcpFd >= 0 && FD_ISSET (xdmcpFd, &reads))
 		ProcessRequestSocket (xdmcpFd);
-#endif
+# endif
 	    if (chooserFd >= 0 && FD_ISSET (chooserFd, &reads))
 	    {
-#ifdef ISC
+# ifdef ISC
 	        if (!ChildReady) {
 	           WaitForSomething ();
                 } else
-#endif
+# endif
 		ProcessChooserSocket (chooserFd);
 		FD_CLR(chooserFd, &reads);
 	    }
-#if defined(IPv6) && defined(AF_INET6)
+# if defined(IPv6) && defined(AF_INET6)
 	    if (chooserFd6 >= 0 && FD_ISSET (chooserFd6, &reads))
 	    {
-#ifdef ISC
+#  ifdef ISC
 	        if (!ChildReady) {
 	           WaitForSomething ();
                 } else
-#endif
+#  endif
 		ProcessChooserSocket (chooserFd6);
 		FD_CLR(chooserFd6, &reads);
 	    }
-#endif 
+# endif
 	    ProcessListenSockets(&reads);
 	}
 	if (ChildReady)
@@ -481,10 +474,10 @@ direct_query_respond (
     xdmOpCode	    type,
     int		    fd)
 {
-    ARRAYofARRAY8   queryAuthenticationNames;
+    ARRAYofARRAY8   queryAuthenticationNames = {0, NULL};
     int		    expectedLen;
     int		    i;
-    
+
     if (!XdmcpReadARRAYofARRAY8 (&buffer, &queryAuthenticationNames))
 	return;
     expectedLen = 1;
@@ -528,7 +521,7 @@ NetworkAddressToName(
     switch (connectionType)
     {
     case FamilyInternet:
-#if defined(IPv6) && defined(AF_INET6)
+# if defined(IPv6) && defined(AF_INET6)
     case FamilyInternet6:
 	{
 	    CARD8		*data;
@@ -570,27 +563,26 @@ NetworkAddressToName(
 
 	    localhost = localHostname ();
 
-	    /* 
-	     * protect against bogus host names 
+	    /*
+	     * protect against bogus host names
 	     */
-	    if (hostname && hostname[0] && (hostname[0] != '.') 
+	    if (hostname && hostname[0] && (hostname[0] != '.')
 			&& !multiHomed)
 	    {
 		if (!strcmp (localhost, hostname))
 		{
-		    if (!getString (name, 10)) {
+		    if (asprintf(&name, ":%d", displayNumber) < 0) {
 			if (ai)
 			    freeaddrinfo(ai);
 			return NULL;
 		    }
-		    sprintf (name, ":%d", displayNumber);
 		}
 		else
 		{
 		    if (removeDomainname)
 		    {
 		    	char    *localDot, *remoteDot;
-    
+
 			/* check for a common domain name.  This
 			 * could reduce names by recognising common
 			 * super-domain names as well, but I don't think
@@ -608,12 +600,12 @@ NetworkAddressToName(
 			}
 		    }
 
-		    if (!getString (name, strlen (hostname) + 10)) {
+		    if (asprintf (&name, "%s:%d",
+				  hostname, displayNumber) < 0) {
 			if (ai)
 			    freeaddrinfo(ai);
 			return NULL;
 		    }
-		    sprintf (name, "%s:%d", hostname, displayNumber);
 		}
 	    }
 	    else
@@ -625,11 +617,11 @@ NetworkAddressToName(
 		}
 		if (multiHomed) {
 		    if (connectionType == FamilyInternet) {
-			data = (CARD8 *) 
+			data = (CARD8 *)
 			  &((struct sockaddr_in *)originalAddress)->
 			  sin_addr;
 		    } else {
-			data = (CARD8 *) 
+			data = (CARD8 *)
 			  &((struct sockaddr_in6 *)originalAddress)->sin6_addr;
 		    }
 		}
@@ -638,14 +630,14 @@ NetworkAddressToName(
 		    if (ai)
 			freeaddrinfo(ai);
 		    return NULL;
-		} 
-		sprintf(name + strlen(name), ":%d", displayNumber);
+		}
+		snprintf(name + strlen(name), 10, ":%d", displayNumber);
 	    }
 	    if (ai)
 		freeaddrinfo(ai);
 	    return name;
 	}
-#else /* IPv6 */
+# else /* IPv6 */
 	{
 	    CARD8		*data;
 	    struct hostent	*hostent;
@@ -664,25 +656,25 @@ NetworkAddressToName(
 
 	    localhost = localHostname ();
 
-	    /* 
-	     * protect against bogus host names 
+	    /*
+	     * protect against bogus host names
 	     */
 	    if (hostent && hostent->h_name && hostent->h_name[0]
-			&& (hostent->h_name[0] != '.') 
+			&& (hostent->h_name[0] != '.')
 			&& !multiHomed)
 	    {
 		if (!strcmp (localhost, hostent->h_name))
 		{
-		    if (!getString (name, 10))
-			return 0;
-		    sprintf (name, ":%d", displayNumber);
+		    if (asprintf(&name, ":%d", displayNumber) < 0) {
+			return NULL;
+		    }
 		}
 		else
 		{
 		    if (removeDomainname)
 		    {
 		    	char    *localDot, *remoteDot;
-    
+
 			/* check for a common domain name.  This
 			 * could reduce names by recognising common
 			 * super-domain names as well, but I don't think
@@ -700,28 +692,31 @@ NetworkAddressToName(
 			}
 		    }
 
-		    if (!getString (name, strlen (hostent->h_name) + 10))
-			return 0;
-		    sprintf (name, "%s:%d", hostent->h_name, displayNumber);
+		    if (asprintf(&name, "%s:%d",
+				 hostent->h_name, displayNumber) < 0) {
+			return NULL;
+		    }
 		}
 	    }
 	    else
 	    {
-		if (!getString (name, 25))
-		    return 0;
 		if (multiHomed)
 		    data = (CARD8 *) &((struct sockaddr_in *)originalAddress)->
 				sin_addr.s_addr;
-		sprintf(name, "%d.%d.%d.%d:%d",
-			data[0], data[1], data[2], data[3], displayNumber);
+
+		if (asprintf(&name, "%d.%d.%d.%d:%d",
+			     data[0], data[1], data[2], data[3],
+			     displayNumber) < 0) {
+		    return NULL;
+		}
 	    }
 	    return name;
 	}
-#endif /* IPv6 */
-#ifdef DNET
+# endif /* IPv6 */
+# ifdef DNET
     case FamilyDECnet:
 	return NULL;
-#endif /* DNET */
+# endif /* DNET */
     default:
 	return NULL;
     }
@@ -735,21 +730,15 @@ forward_respond (
     int			length,
     int			fd)
 {
-    ARRAY8	    clientAddress;
-    ARRAY8	    clientPort;
-    ARRAYofARRAY8   authenticationNames;
+    ARRAY8	    clientAddress = {0, NULL};
+    ARRAY8	    clientPort = {0, NULL};
+    ARRAYofARRAY8   authenticationNames = {0, NULL};
     struct sockaddr *client;
     int		    clientlen;
     int		    expectedLen;
     int		    i;
-    
+
     Debug ("Forward respond %d\n", length);
-    clientAddress.length = 0;
-    clientAddress.data = NULL;
-    clientPort.length = 0;
-    clientPort.data = NULL;
-    authenticationNames.length = 0;
-    authenticationNames.data = NULL;
     if (XdmcpReadARRAY8 (&buffer, &clientAddress) &&
 	XdmcpReadARRAY8 (&buffer, &clientPort) &&
 	XdmcpReadARRAYofARRAY8 (&buffer, &authenticationNames))
@@ -773,7 +762,7 @@ forward_respond (
 	    Debug ("\n");
     	    switch (from->sa_family)
     	    {
-#ifdef AF_INET
+# ifdef AF_INET
 	    case AF_INET:
 		{
 		    struct sockaddr_in	in_addr;
@@ -784,9 +773,9 @@ forward_respond (
 			goto badAddress;
 		    }
 		    bzero ((char *) &in_addr, sizeof (in_addr));
-#ifdef BSD44SOCKETS
+#  ifdef BSD44SOCKETS
 		    in_addr.sin_len = sizeof(in_addr);
-#endif
+#  endif
 		    in_addr.sin_family = AF_INET;
 		    memmove( &in_addr.sin_addr, clientAddress.data, 4);
 		    memmove( (char *) &in_addr.sin_port, clientPort.data, 2);
@@ -796,8 +785,8 @@ forward_respond (
 			       FORWARD_QUERY, fd);
 		}
 		break;
-#endif
-#if defined(IPv6) && defined(AF_INET6)
+# endif
+# if defined(IPv6) && defined(AF_INET6)
 	    case AF_INET6:
 		{
 		    struct sockaddr_in6	in6_addr;
@@ -808,11 +797,11 @@ forward_respond (
 			goto badAddress;
 		    }
 		    bzero ((char *) &in6_addr, sizeof (in6_addr));
-#ifdef SIN6_LEN
+#  ifdef SIN6_LEN
 		    in6_addr.sin6_len = sizeof(in6_addr);
-#endif
+#  endif
 		    in6_addr.sin6_family = AF_INET6;
-		    memmove(&in6_addr,clientAddress.data,clientAddress.length);
+		    memmove(&in6_addr.sin6_addr,clientAddress.data,clientAddress.length);
 		    memmove((char *) &in6_addr.sin6_port, clientPort.data, 2);
 		    client = (struct sockaddr *) &in6_addr;
 		    clientlen = sizeof (in6_addr);
@@ -820,8 +809,8 @@ forward_respond (
 			       FORWARD_QUERY, fd);
 		}
 		break;
-#endif
-#ifdef AF_UNIX
+# endif
+# ifdef AF_UNIX
 	    case AF_UNIX:
 		{
 		    struct sockaddr_un	un_addr;
@@ -833,25 +822,25 @@ forward_respond (
 		    memmove( un_addr.sun_path, clientAddress.data, clientAddress.length);
 		    un_addr.sun_path[clientAddress.length] = '\0';
 		    client = (struct sockaddr *) &un_addr;
-#if defined(BSD44SOCKETS) && !defined(Lynx) && defined(UNIXCONN)
+#  if defined(BSD44SOCKETS) && defined(UNIXCONN)
 		    un_addr.sun_len = strlen(un_addr.sun_path);
 		    clientlen = SUN_LEN(&un_addr);
-#else
+#  else
 		    clientlen = sizeof (un_addr);
-#endif
+#  endif
 		    all_query_respond (client, clientlen, &authenticationNames,
 			       FORWARD_QUERY, fd);
 		}
 		break;
-#endif
-#ifdef AF_CHAOS
+# endif
+# ifdef AF_CHAOS
 	    case AF_CHAOS:
 		goto badAddress;
-#endif
-#ifdef AF_DECnet
+# endif
+# ifdef AF_DECnet
 	    case AF_DECnet:
 		goto badAddress;
-#endif
+# endif
     	    }
 	}
 	else
@@ -919,7 +908,7 @@ send_unwilling (
 
 static unsigned long	globalSessionID;
 
-#define NextSessionID()    (++globalSessionID)
+# define NextSessionID()    (++globalSessionID)
 
 void init_session_id(void)
 {
@@ -929,7 +918,7 @@ void init_session_id(void)
      */
     globalSessionID = (time((Time_t *)0)&0x7fff) * 16000;
 }
-    
+
 static ARRAY8 outOfMemory = { (CARD16) 13, (CARD8Ptr) "Out of memory" };
 static ARRAY8 noValidAddr = { (CARD16) 16, (CARD8Ptr) "No valid address" };
 static ARRAY8 noValidAuth = { (CARD16) 22, (CARD8Ptr) "No valid authorization" };
@@ -943,36 +932,21 @@ request_respond (
     int		    fd)
 {
     CARD16	    displayNumber;
-    ARRAY16	    connectionTypes;
-    ARRAYofARRAY8   connectionAddresses;
-    ARRAY8	    authenticationName;
-    ARRAY8	    authenticationData;
-    ARRAYofARRAY8   authorizationNames;
-    ARRAY8	    manufacturerDisplayID;
+    ARRAY16	    connectionTypes = {0, NULL};
+    ARRAYofARRAY8   connectionAddresses = {0, NULL};
+    ARRAY8	    authenticationName = {0, NULL};
+    ARRAY8	    authenticationData = {0, NULL};
+    ARRAYofARRAY8   authorizationNames = {0, NULL};
+    ARRAY8	    manufacturerDisplayID = {0, NULL};
     ARRAY8Ptr	    reason = NULL;
     int		    expectlen;
     int		    i, j;
     struct protoDisplay  *pdpy = NULL;
-    ARRAY8	    authorizationName, authorizationData;
+    ARRAY8	    authorizationName = {0, NULL},
+		    authorizationData = {0, NULL};
     ARRAY8Ptr	    connectionAddress;
 
     Debug ("Request respond %d\n", length);
-    connectionTypes.length = 0;
-    connectionTypes.data = NULL;
-    connectionAddresses.length = 0;
-    connectionAddresses.data = NULL;
-    authenticationName.length = 0;
-    authenticationName.data = NULL;
-    authenticationData.length = 0;
-    authenticationData.data = NULL;
-    authorizationNames.length = 0;
-    authorizationNames.data = NULL;
-    authorizationName.length = 0;
-    authorizationName.data = NULL;
-    authorizationData.length = 0;
-    authorizationData.data = NULL;
-    manufacturerDisplayID.length = 0;
-    manufacturerDisplayID.data = NULL;
     if (XdmcpReadCARD16 (&buffer, &displayNumber) &&
 	XdmcpReadARRAY16 (&buffer, &connectionTypes) &&
 	XdmcpReadARRAYofARRAY8 (&buffer, &connectionAddresses) &&
@@ -1019,7 +993,7 @@ request_respond (
 		reason = &noValidAddr;
 		goto decline;
 	    }
-	
+
 	    /* The Manager considers this a new session */
 	    connectionAddress = &connectionAddresses.data[i];
 	    pdpy = NewProtoDisplay ((XdmcpNetaddr) from, fromlen, displayNumber,
@@ -1122,7 +1096,7 @@ send_accept (
     XdmcpWriteARRAY8 (&buffer, authorizationData);
     XdmcpFlush (fd, &buffer, (XdmcpNetaddr) to, tolen);
 }
-   
+
 static void
 send_decline (
     struct sockaddr *to,
@@ -1157,19 +1131,18 @@ manage (
 {
     CARD32		sessionID;
     CARD16		displayNumber;
-    ARRAY8		displayClass;
+    ARRAY8		displayClass = {0, NULL};
     int			expectlen;
     struct protoDisplay	*pdpy;
     struct display	*d;
     char		*name = NULL;
     char		*class = NULL;
     XdmcpNetaddr	from_save;
-    ARRAY8		clientAddress, clientPort;
+    ARRAY8		clientAddress = {0, NULL},
+			clientPort = {0, NULL};
     CARD16		connectionType;
 
     Debug ("Manage %d\n", length);
-    displayClass.data = NULL;
-    displayClass.length = 0;
     if (XdmcpReadCARD32 (&buffer, &sessionID) &&
 	XdmcpReadCARD16 (&buffer, &displayNumber) &&
 	XdmcpReadARRAY8 (&buffer, &displayClass))
@@ -1194,7 +1167,7 @@ manage (
 	     * If all this is true, then we have a duplicate request that
 	     * can be ignored.
 	     */
-	    if (!pdpy 
+	    if (!pdpy
 		&& (d = FindDisplayByAddress((XdmcpNetaddr) from, fromlen, displayNumber))
 		&& d->sessionID == sessionID) {
 		     Debug("manage: got duplicate pkt, ignoring\n");
@@ -1215,7 +1188,7 @@ manage (
 		   name, (char *)pdpy->connectionAddress.data);
 	    if (!name)
 	    {
-		send_failed (from, fromlen, "(no name)", sessionID, 
+		send_failed (from, fromlen, "(no name)", sessionID,
 		  "out of memory", fd);
 		goto abort;
 	    }
@@ -1228,7 +1201,7 @@ manage (
 	    class = malloc (displayClass.length + 1);
 	    if (!class)
 	    {
-		send_failed (from, fromlen, name, sessionID, 
+		send_failed (from, fromlen, name, sessionID,
 		  "out of memory", fd);
 		goto abort;
 	    }
@@ -1254,7 +1227,7 @@ manage (
 	    if (!d)
 	    {
 		free ((char *) from_save);
-		send_failed (from, fromlen, name, sessionID, 
+		send_failed (from, fromlen, name, sessionID,
 		  "out of memory", fd);
 		goto abort;
 	    }
@@ -1313,7 +1286,7 @@ SendFailed (
     char	    *reason)
 {
     Debug ("Display start failed, sending Failed\n");
-    send_failed ((struct sockaddr *)(d->from), d->fromlen, d->name, 
+    send_failed ((struct sockaddr *)(d->from), d->fromlen, d->name,
       d->sessionID, reason, d->xdmcpFd);
 }
 
@@ -1330,8 +1303,8 @@ send_failed (
     XdmcpHeader	header;
     ARRAY8	status;
 
-    sprintf (buf, "Session %ld failed for display %.100s: %.100s",
-	     (long) sessionID, name, reason);
+    snprintf (buf, sizeof(buf), "Session %ld failed for display %.100s: %s",
+	      (long) sessionID, name, reason);
     Debug ("Send failed %ld %s\n", (long) sessionID, buf);
     status.length = strlen (buf);
     status.data = (CARD8Ptr) buf;
@@ -1416,24 +1389,24 @@ NetworkAddressToHostname (
     switch (connectionType)
     {
     case FamilyInternet:
-#if defined(IPv6) && defined(AF_INET6)
+# if defined(IPv6) && defined(AF_INET6)
     case FamilyInternet6:
-#endif
+# endif
 	{
 	    struct hostent	*hostent = NULL;
-#if defined(IPv6) && defined(AF_INET6)
+# if defined(IPv6) && defined(AF_INET6)
 	    char dotted[INET6_ADDRSTRLEN];
-#else
+# else
 	    char dotted[20];
-#endif
+# endif
 	    char *local_name = "";
 	    int af_type;
 
-#if defined(IPv6) && defined(AF_INET6)
+# if defined(IPv6) && defined(AF_INET6)
 	    if (connectionType == FamilyInternet6)
 		af_type = AF_INET6;
 	    else
-#endif
+# endif
 		af_type = AF_INET;
 
 	    hostent = gethostbyaddr ((char *)connectionAddress->data,
@@ -1441,7 +1414,7 @@ NetworkAddressToHostname (
 
 	    if (hostent) {
 		/* check for DNS spoofing */
-#if defined(IPv6) && defined(AF_INET6)
+# if defined(IPv6) && defined(AF_INET6)
 		struct addrinfo	*ai = NULL, *nai;
 		if (getaddrinfo(hostent->h_name, NULL, NULL, &ai) == 0) {
 		    for (nai = ai; nai != NULL; nai = nai->ai_next) {
@@ -1459,7 +1432,7 @@ NetworkAddressToHostname (
 			    break;
 		    }
 		    if (nai == NULL) {
-			inet_ntop(af_type, connectionAddress->data, 
+			inet_ntop(af_type, connectionAddress->data,
 			  dotted, sizeof(dotted));
 
 			LogError("Possible DNS spoof attempt %s->%s.\n", dotted,
@@ -1472,7 +1445,7 @@ NetworkAddressToHostname (
 		} else {
 		    hostent = NULL;
 		}
-#else
+# else
 		char *s = strdup(hostent->h_name); /* fscking non-reentrancy of getXXX() */
 		if ((hostent = gethostbyname(s))) {
 			if (memcmp((char*)connectionAddress->data, hostent->h_addr,
@@ -1484,41 +1457,39 @@ NetworkAddressToHostname (
 			}
 		}
 		free(s);
-#endif
+# endif
 	    }
 
 	    if (!hostent) {
 		/* can't get name, so use emergency fallback */
-#if defined(IPv6) && defined(AF_INET6)
-		inet_ntop(af_type, connectionAddress->data, 
+# if defined(IPv6) && defined(AF_INET6)
+		inet_ntop(af_type, connectionAddress->data,
 		  	  dotted, sizeof(dotted));
-#else
-		sprintf(dotted, "%d.%d.%d.%d",
-			connectionAddress->data[0],
-			connectionAddress->data[1],
-			connectionAddress->data[2],
-			connectionAddress->data[3]);
-#endif
+# else
+		snprintf(dotted, sizeof(dotted), "%d.%d.%d.%d",
+			 connectionAddress->data[0],
+			 connectionAddress->data[1],
+			 connectionAddress->data[2],
+			 connectionAddress->data[3]);
+# endif
 		local_name = dotted;
 		LogError ("Cannot convert Internet address %s to host name\n",
 			  dotted);
 	    }
-	    if (!getString (name, strlen (local_name)))
-		break;
-	    strcpy (name, local_name);
+	    name = strdup (local_name);
 	    break;
 	}
-#ifdef DNET
+# ifdef DNET
     case FamilyDECnet:
 	break;
-#endif /* DNET */
+# endif /* DNET */
     default:
 	break;
     }
     return name;
 }
 
-#if 0
+# if 0
 static int
 HostnameToNetworkAddress (
 char	    *name,
@@ -1539,10 +1510,10 @@ ARRAY8Ptr   connectionAddress)
 	    memmove( connectionAddress->data, hostent->h_addr, hostent->h_length);
 	    return TRUE;
 	}
-#ifdef DNET
+#  ifdef DNET
     case FamilyDECnet:
 	return FALSE;
-#endif
+#  endif
     }
     return FALSE;
 }
@@ -1583,10 +1554,10 @@ CARD16Ptr   displayNumber)
 	dnet = TRUE;
 	colon++;
     }
-#ifndef DNETCONN
+#  ifndef DNETCONN
     if (dnet)
 	return FALSE;
-#endif
+#  endif
     display_number = colon + 1;
     while (*display_number && *display_number != '.')
     {
@@ -1596,11 +1567,11 @@ CARD16Ptr   displayNumber)
     if (display_number == colon + 1)
 	return FALSE;
     number = atoi (colon + 1);
-#ifdef DNETCONN
+#  ifdef DNETCONN
     if (dnet)
 	connectionType = FamilyDECnet;
     else
-#endif
+#  endif
 	connectionType = FamilyInternet;
     if (!HostnameToNetworkAddress (hostname, connectionType, connectionAddress))
 	return FALSE;
@@ -1608,6 +1579,6 @@ CARD16Ptr   displayNumber)
     *connectionTypep = connectionType;
     return TRUE;
 }
-#endif
+# endif
 
 #endif /* XDMCP */
